from django.conf.urls import url, include
from .views import Index
from .views import NotesListCreateView, NotesListDetailView, NotesListUpdateView, NotesListDeleteView
from .views import NoteCreateView, NoteDetailView, NoteUpdateView, NoteDeleteView

urlpatterns = [
    url(r'^$', Index.as_view(), name='index'),
    url(r'^list/', include([
        url('^add/$', NotesListCreateView.as_view(), name='list-add'),
        url(r'^(?P<notes_list_id>\d+)/', include([
            url(r'^$', NotesListDetailView.as_view(), name='list-detail'),
            url(r'^update/$', NotesListUpdateView.as_view(), name='list-update'),
            url(r'^delete/$', NotesListDeleteView.as_view(), name='list-delete'),
            url(r'^item/', include([
                url(r'add/$', NoteCreateView.as_view(), name='list-item-add'),
                url(r'(?P<note_id>\d+)/', include([
                    url(r'^$', NoteDetailView.as_view(), name='list-item-detail'),
                    url(r'^update/$', NoteUpdateView.as_view(), name='list-item-update'),
                    url(r'^delete/$', NoteDeleteView.as_view(), name='list-item-delete'),
                ]))
            ]))
        ]))
    ])),
]



# from django.shortcuts import render
from django.contrib.auth.mixins import LoginRequiredMixin
from django.views.generic import ListView, DetailView
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.urls import reverse_lazy, reverse
from .models import NotesList, Note


class Index(LoginRequiredMixin, ListView):
    model = NotesList
    context_object_name = 'notes_lists'
    template_name = 'note/index.html'

    def get_queryset(self):
        return self.model.objects.filter(user=self.request.user)


class CRUDViewCommonMixin:
    model = None
    pk_url_kwarg = ''
    context_object_name = ''


class NotesListCRUDViewCommonMixin(CRUDViewCommonMixin):
    model = NotesList
    pk_url_kwarg = 'notes_list_id'
    context_object_name = 'notes_list'


class NotesListCreateView(LoginRequiredMixin, NotesListCRUDViewCommonMixin, CreateView):
    fields = ['name']
    template_name = 'note/list/create.html'

    def form_valid(self, form):
        form.instance.user = self.request.user
        return super().form_valid(form)


class NotesListDetailView(LoginRequiredMixin, NotesListCRUDViewCommonMixin, DetailView):
    template_name = 'note/list/detail.html'


class NotesListUpdateView(LoginRequiredMixin, NotesListCRUDViewCommonMixin, UpdateView):
    fields = ['name']
    template_name = 'note/list/update.html'


class NotesListDeleteView(LoginRequiredMixin, NotesListCRUDViewCommonMixin, DeleteView):
    template_name = 'note/list/delete.html'
    success_url = reverse_lazy('note:index')


class NoteCRUDViewCommonMixin(CRUDViewCommonMixin):
    model = Note
    pk_url_kwarg = 'note_id'
    context_object_name = 'note'


class NoteCreateView(LoginRequiredMixin, NoteCRUDViewCommonMixin, CreateView):
    fields = ['title', 'text']
    template_name = 'note/list/item/create.html'

    def form_valid(self, form):
        form.instance.notes_list = NotesList.objects.get(id=self.kwargs['notes_list_id'])
        return super().form_valid(form)

    def get_success_url(self):
        return reverse('note:list-detail', kwargs=self.kwargs)


class NoteDetailView(LoginRequiredMixin, NoteCRUDViewCommonMixin, DetailView):
    template_name = 'note/list/item/detail.html'


class NoteUpdateView(LoginRequiredMixin, NoteCRUDViewCommonMixin, UpdateView):
    fields = ['title', 'text', 'done']
    template_name = 'note/list/update.html'

    def get_success_url(self):
        return reverse('note:list-detail', kwargs={'notes_list_id': self.kwargs['notes_list_id']})


class NoteDeleteView(LoginRequiredMixin, NoteCRUDViewCommonMixin, DeleteView):
    template_name = 'note/list/item/delete.html'

    def get_success_url(self):
        return reverse('note:list-detail', kwargs={'notes_list_id': self.kwargs['notes_list_id']})



from django.contrib import admin
from.models import NotesList, Note


@admin.register(NotesList)
class TodoListAdmin(admin.ModelAdmin):
    search_fields = ('name',)
    date_hierarchy = 'created'
    ordering = ('-modified',)


@admin.register(Note)
class TodoListItemAdmin(admin.ModelAdmin):
    search_fields = ('title', )
    list_filter = ('done', )



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-note',
    version='1.1.1',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Django==1.11.6',
    ],
    license='GPL-3.0',
    description='A simple Django app to conduct Web-based note lists.',
    long_description=README,
    url='https://www.example.com/',
    author='Skochinskyi Dmytro',
    author_email='skodmy@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



"""
note app models goes here.
"""
from django.db import models
from django.conf import settings
from django.urls import reverse


class NotesList(models.Model):
    """
    A class that represents a user's note list.
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    name = models.CharField(max_length=120)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    def __str__(self):
        return '{}, created: {}, modified: {}'.format(
            self.name,
            self.created,
            self.modified
        )

    @property
    def items(self):
        for note in Note.objects.filter(notes_list=self):
            yield note

    @property
    def completed(self):
        """
        Checks if all items for self are checked.

        :return: True if all are True, False otherwise.
        """
        return all((note.done for note in self.items))

    def get_absolute_url(self):
        return reverse('note:list-detail', kwargs={'notes_list_id': self.id})


class Note(models.Model):
    """
    A class which objects represent note list items.
    """
    notes_list = models.ForeignKey(NotesList)
    title = models.CharField(max_length=150)
    text = models.TextField(blank=True)
    done = models.BooleanField(default=False)

    def __str__(self):
        return '{} from {} is {}'.format(self.title, self.notes_list.name, 'done' if self.done else 'undone')

    def get_absolute_url(self):
        return reverse(
            'note:list-item-detail',
            kwargs={'note_list_id': self.notes_list.id, 'note_list_item_id': self.id}
        )



from django.test import TestCase, RequestFactory
from django.conf import settings
from django.apps import apps
from django.contrib.auth.models import AnonymousUser

from .models import NotesList, Note
from .views import Index
from .views import NotesListCreateView
from .views import NoteCreateView, NoteUpdateView, NoteDeleteView


AUTH_USER_MODEL = apps.get_model(settings.AUTH_USER_MODEL)


class NotesListModelTestCase(TestCase):
    def setUp(self):
        self.notes_list = NotesList.objects.create(name='Test list', user=AUTH_USER_MODEL.objects.create())

    def test_datetime_logic(self):
        self.assertLessEqual(self.notes_list.created, self.notes_list.modified)

    def test_completed(self):
        self.assertTrue(self.notes_list.completed)


class NoteModelTestCase(TestCase):
    def setUp(self):
        self.note_list_item = Note.objects.create(
            title='Test item',
            notes_list=NotesList.objects.create(name='Test list', user=AUTH_USER_MODEL.objects.create()),
            text='Lorem ipsum dolor ... sit amet'
        )

    def test_done(self):
        self.assertFalse(self.note_list_item.done)


class ViewTestCase(TestCase):
    def setUp(self):
        self._view = None
        self._request_factory = RequestFactory()
        self._user = AnonymousUser()


class ConcreteUserViewTestCase(ViewTestCase):
    def setUp(self):
        super().setUp()
        self._user = AUTH_USER_MODEL.objects.create()


class IndexViewTestCase(ConcreteUserViewTestCase):
    def setUp(self):
        super().setUp()
        self._view = Index()
        self._view.request = self._request_factory.get('/')
        self._view.request.user = self._user

    def test_get_queryset(self):
        for notes_list in self._view.get_queryset():
            self.assertEqual(notes_list.user, self._user)

    def test_as_view(self):
        response = Index.as_view()(self._view.request)
        self.assertEqual(response.status_code, 200)


class CRUDViewCommonMixinTestCase(TestCase):
    pass


class NotesListCRUDViewCommonMixinTestCase(TestCase):
    pass


class NotesListCreateViewTestCase(ConcreteUserViewTestCase):
    def setUp(self):
        super().setUp()
        self._view = NotesListCreateView()
        self._view.request = self._request_factory.get('/list/add/')
        self._view.request.user = self._user

    def test_form_valid(self):
        form = self._view.get_form_class()({'name': 'test_list'})
        self.assertEqual(self._view.form_valid(form).status_code, 302)

    def test_as_view(self):
        response = NotesListCreateView.as_view()(self._view.request)
        self.assertEqual(response.status_code, 200)


class NotesListDetailViewTestCase(ConcreteUserViewTestCase):
    pass


class NotesListUpdateViewTestCase(ConcreteUserViewTestCase):
    pass


class NotesListDeleteViewTestCase(ConcreteUserViewTestCase):
    pass


class NoteCRUDViewCommonMixinTestCase(ConcreteUserViewTestCase):
    pass


class NoteCreateViewTestCase(ConcreteUserViewTestCase):
    def setUp(self):
        super().setUp()
        self._view = NoteCreateView()
        self._view.request = self._request_factory.get('/list/item/add/')
        self._view.request.user = self._user
        test_notes_list = NotesList.objects.create(name='test note list', user=self._user)
        setattr(self._view, 'kwargs', {'notes_list_id': test_notes_list.id})

    def test_form_valid(self):
        form = self._view.get_form_class()({'title': 'test_list item', 'text': 'test_list item text'})
        self.assertEqual(self._view.form_valid(form).status_code, 302)

    def test_get_success_url(self):
        self.assertNotEqual(self._view.get_success_url(), '')

    def test_as_view(self):
        response = NoteCreateView.as_view()(self._view.request)
        self.assertEqual(response.status_code, 200)


class NoteDetailViewTestCase(ConcreteUserViewTestCase):
    pass


class NoteUpdateViewTestCase(ConcreteUserViewTestCase):
    def setUp(self):
        super().setUp()
        self._view = NoteUpdateView()
        test_notes_list = NotesList.objects.create(name='test note list', user=self._user)
        setattr(self._view, 'kwargs', {'notes_list_id': test_notes_list.id})

    def test_get_success_url(self):
        self.assertNotEqual(self._view.get_success_url(), '')


class NoteDeleteViewTestCase(ConcreteUserViewTestCase):
    def setUp(self):
        super().setUp()
        self._view = NoteDeleteView()
        test_notes_list = NotesList.objects.create(name='test note list', user=self._user)
        setattr(self._view, 'kwargs', {'notes_list_id': test_notes_list.id})

    def test_get_success_url(self):
        self.assertNotEqual(self._view.get_success_url(), '')
